<?php

/**
 * Carerix Wordpress Plugin
 *
 * @category settings
 * @author Bogdan Ivanov
 * @copyright Copyright (c) 2013 Carerix.com (http://www.carerix.com)
 * @link http://www.carerix.com
 * @since Aug 29, 2013 1:08:16 PM
 * @version $Rev$
 */


/**
 * Job class
 * Used to ....
 */
class Entity_Cron_Job extends Entity_Cron {
	
	protected $show = array('publicationID','publicationStart','creationDate', 
        'applicationContactInformation', 
		'companyInformation', 'functionContactInformation', 'introInformation',
		'layoutInstruction', 'notes', 'offerInformation', 'publicationSpecifications',
		'requirementsInformation', 'titleInformation', 'vacancyInformation', 'workLocation',
		'toMedium.code','toVacancy.vacancyID', 'toVacancy.jobTitle', 'publicationStart',
		);
	private $post_id = 0;
	protected $registerLanguage = false;
    

    public function getUrl() {
		$jobsUrl = Environment_Installation::get_publications_feed();
		if(@$this->sourceData['settings']['feed'])
		{
			$jobsUrl .= '?'.$this->sourceData['settings']['feed'];
		}
		if(stripos($jobsUrl, 'medium') === FALSE)
		{
			$extra = array('medium' => 'web');
			$jobsUrl = $this->interfacer->url_merger($jobsUrl, $extra);
		}
        
		if(    $this->registerLanguage 
			&& $this->interfacer->wpml_is_active()
            && strtolower(trim($this->registerLanguage)) != trim($this->interfacer->language_major_current()))
		{
			$jobsUrl = $this->interfacer->url_medium_for_translation($jobsUrl, $this->registerLanguage);
		}

		return $jobsUrl;
	}
	
	public function setRegisterLanguage($language)
	{
		$this->registerLanguage = $language;
	}

	public function process($data) 
	{
        Utils_Cache::cleanEnvironment();
		return $this->processData($data);
	}

    public function getCategory() 
    {
        // get the parent category for this jobs source based on medium
        $jobsUrl = $this->getUrl();
        $data = parse_url($jobsUrl);
        $query = array();
        $category = false;
        if(isset($data['query'])) 
        {
            parse_str($data['query'], $query);
        }
        
        if(isset($query['medium']) && $query['medium'] !== 'web' && stripos($query['medium'], 'web') !==0 ) 
        {
            if(stripos($query['medium'], '-') !== FALSE)
            {
                $v = explode('-', $query['medium']);
                $category = $v[0];
            }
            else
            {
                $category = $query['medium'];
            }
        }
        
        if(isset($query['medium']) && $query['medium'] == 'web')
        {
            $category = 'Publications';
        }
        
        return $category;
    }
    
	private function processData($data)
	{
		$items = $data->channel->item;
		
		$itemsTable = $this->getArrayOfItems($items);
        
        $customFields = $this->getCustomFieldsData($items);
		$listOfFragments = array();

        $category = $this->getCategory();
        
        
		/**
		 * for each item in the table,
		 * create a writer,
		 * write,
		 * collect a fragment
		 */
        $count = 0;
		foreach($itemsTable as $item)
		{
			$job = new Entity_Writer_Job($item, $customFields[$count]);
            $job->setSourceData($this->sourceData);
			if($category) $job->setCategoryKey ($category);
			if($this->registerLanguage !== FALSE)
			{
				$job->setCategoryLanguage($this->registerLanguage);
				$job->setTypeSuffix($this->registerLanguage);
			}
			
			$id = $job->write();
			array_push($listOfFragments, $job->getFragment($id));
			$this->interfacer->entity_set_property( $id, 
													'source', 
													$this->sourceData['id']);
            $this->interfacer->entity_set_property($id, 'source_cron', $this->sourceData['id']);
            
			/**
			 * register this post for the set language in the WPML table
			 */
			if($this->registerLanguage !== FALSE)
			{
				$trid = trim($item['vacancyID']);
				$this->interfacer->wpml_register_translation($id, $trid, 'post_post', $this->registerLanguage);
			}
            $count++;
            $job = null;
		}
		
		if(count($listOfFragments) > 0 )
		{
			/**
			* launch the collector for all the items from this source.
			* creates a page in wordpress
			*/
			$collector = new Entity_Writer_Collector(
                    $this->sourceData['name'], 
                    $this->sourceData['type'], 
					$this->sourceData['id'], 
                    $listOfFragments, 
                    @$this->sourceData['settings']['introtext']);
			/**
			 * register this collector page for the set language in the WPML table
			 */
			if($this->registerLanguage !== FALSE)
			{
				$collector->setTypeSuffix($this->registerLanguage);
			}
			$element_id = $collector->write(); 
			$this->setPostID($element_id);
			if($this->registerLanguage !== FALSE)
			{
				$trid = (int)$this->sourceData['id'] * 1000000;
				$type = 'page_page';
				$this->interfacer->wpml_register_translation($element_id, $trid, $type, $this->registerLanguage);
			}
            $collector = null;
		}
        
        return count($itemsTable);
	}
    
    public function setPostID($id) {
        $this->post_id = $id;
    }
    
    public function getPostID() {
        return $this->post_id;
    }
	
	/**
	 * Formats the retrieved xml into a list of tables
	 * 
	 * 
	 * @param SimpleXML $data
	 * @return array
	 */
	private function getArrayOfItems($data)
	{
		$items = array();
		
		$collectors = array(
                    'guid', 'pubDate', 'title', 
                    'introInformation',  // 1
                    'companyInformation',  // 2
                    'vacancyInformation', // 3
                    'requirementsInformation', // 4
                    'offerInformation', // 5
                    'functionContactInformation', // 6
                    'applicationContactInformation', // 7
                    'category',   'workLocation',
                    'rawTitle', 'titleWorkLocation',
                    'vacancyID', 'country1', 'country2', 'country3',
                    'region1', 'region2', 'region3', 'medium', 'educations', 'procedure',
                    'function0', 'function1',
                    'toWorkLevelNode',
                    'ownerName'
		);
		
        
		foreach($data as $item)
		{
			$newItem = array();
			
			foreach($collectors as $collector)
			{
                if($collector == 'educations')
                {
                    $educations = array();
                    foreach($item->$collector->education as $education)
                    {
                        $education = (string)$education;
                        array_push($educations, $education);
                    }
                    $newItem[$collector] = $educations;
                    
                    $requiredEducations = array();
                    foreach($item->$collector->required as $requiredEducation)
                    {
                        $required = (string)$requiredEducation;
                        array_push($requiredEducations, $required);
                    }
                    $newItem['requiredEducations'] = $requiredEducations;
                }
                
				if(isset($item->$collector) 
                    && strlen(trim((string)$item->$collector)) > 0 
                    )
				{
                    $name_collector = preg_replace('/\s+/u'  , ' | ',trim(strip_tags((string)$item->$collector)));
                    $name_collector = htmlentities($name_collector, null, 'utf-8');
                    $name_collector = str_replace('&nbsp;', '', $name_collector);
                    $name_collector = str_replace('&#160;', '', $name_collector); 

                    if(strlen($name_collector) == 0) continue;
                    
					$newItem[$collector] = preg_replace('/\s+/u', ' ', trim((string)$item->$collector));
                    
                    if($collector == 'title') {
                        $newItem[$collector] = preg_replace('/\s+/u', ' ', trim((string)$item->rawTitle));
                        $typeOfTitle = $this->sourceData['settings']['title_compose'];
                        if($typeOfTitle == 'title') {
                            $real_title = $newItem[$collector];
                        } else {
                            $location = preg_replace('/\s+/u', ' ', trim((string)$item->titleWorkLocation));
                            $real_title = implode(' - ', array($newItem[$collector], $location));
                        }
                        $newItem[$collector] = $real_title;
                    }
                    
				}
			}
			
			if(!empty($newItem)) array_push ($items, $newItem);
		}
		
        
		return $items;
	}

    private function getCustomFieldsData($data) {
        
        $customFieldCollectors = array(
            'vacancyID', 'title', 'pubDate', 'workLocation', 'contact', 'company',
            'medium', 'introInformation', 'companyInformation', 'vacancyInformation',
            'requirementsInformation', 'offerInformation', 'functionContactInformation',
            'applicationContactInformation', 'startDate', 'endDate', 'guid', 'postalCode'
        );
        $items = array();
        
        foreach($data as $item) {
            $newItem = array();
            foreach($customFieldCollectors as $collector) {
                
                if(isset($item->$collector) && strlen(trim((string)$item->$collector)) > 0 ) 
                {
                    $newItem[$collector] = preg_replace('/\s+/u', ' ', trim((string)$item->$collector));
                }
            }
            if(!empty($newItem)) array_push ($items, $newItem);
        }
        
        return $items;
    }
}